{% extends "allianceauth/base.html" %}
{% load bootstrap %}
{% load static %}
{% load i18n %}

{% block page_title %}{% translate "Personal fatlink statistics" %}{% endblock page_title %}

{% block content %}
    <div class="col-lg-12">
        <h1 class="page-header text-center">{% blocktrans %}Participation data statistics for {{ year }}{% endblocktrans %}
            <div class="text-right">
                <a href="{% url 'fatlink:personal_statistics_year' previous_year %}" class="btn btn-info">{% translate "Previous year" %}</a>
                {%  if next_year %}
                <a href="{% url 'fatlink:personal_statistics_year' next_year  %}" class="btn btn-info">{% translate "Next year" %}</a>
                {%  endif %}
            </div>
        </h1>
        <div class="col-lg-2 col-lg-offset-5">
        <table class="table table-responsive">
            <tr>
                <th class="col-md-2 text-center">{% translate "Month" %}</th>
                <th class="col-md-2 text-center">{% translate "Fats" %}</th>
            </tr>
            {% for monthnr, month, n_fats in monthlystats %}
            <tr>
                <td class="text-center">
                    <a href="{% url 'fatlink:personal_statistics_month' year monthnr %}">
                        {{ month }}
                    </a>
                </td>
                <td class="text-center">{{ n_fats }}</td>
            </tr>
            {% endfor %}
        </table>
        </div>
    </div>
{% endblock content %}
