# Generated by Django 1.10.5 on 2017-03-27 03:29

from django.db import migrations
import sortedm2m.fields
from sortedm2m.operations import AlterSortedManyToManyField


class Migration(migrations.Migration):

    dependencies = [
        ('hrapplications', '0004_make_strings_more_stringy'),
    ]

    operations = [
        AlterSortedManyToManyField(
            model_name='applicationform',
            name='questions',
            field=sortedm2m.fields.SortedManyToManyField(help_text=None, to='hrapplications.ApplicationQuestion'),
        ),
    ]
