{% load evelinks %}

<tr>
    <td>
        {{ type }}: {{ name }}
    </td>
    <td class="text-right">
        <img src="{{ user.profile.main_character|character_portrait_url:32 }}" class="img-circle">
    </td>
    <td>
        <strong>{{ user }}<br></strong>
        {{ user.profile.main_character.character_name }}
    </td>
    <td class="text-left">
        {% if user.profile.main_character %}
            <a href="{{ user.profile.main_character|dotlan_corporation_url }}" target="_blank">
                {{ user.profile.main_character.corporation_name }}
            </a><br>
            {{ user.profile.main_character.alliance_name|default_if_none:"" }}
        {% else %}
            (unknown)
        {% endif %}

    </td>
</tr>
