# Generated by Django 3.2.9 on 2021-11-11 15:56

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('auth', '0012_alter_user_first_name_max_length'),
        ('authentication', '0019_merge_20211026_0919'),
        ('groupmanagement', '0016_remove_grouprequest_status_field'),
    ]

    operations = [
        migrations.AlterField(
            model_name='authgroup',
            name='group_leader_groups',
            field=models.ManyToManyField(blank=True, help_text='Members of leader groups can process requests for this group. Use the <code>auth.group_management</code> permission to allow a user to manage all groups.<br>', related_name='leads_group_groups', to='auth.Group'),
        ),
        migrations.AlterField(
            model_name='authgroup',
            name='group_leaders',
            field=models.ManyToManyField(blank=True, help_text='Group leaders can process requests for this group. Use the <code>auth.group_management</code> permission to allow a user to manage all groups.<br>', related_name='leads_groups', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='authgroup',
            name='open',
            field=models.BooleanField(default=False, help_text='Group is open and users will be automatically added upon request.<br>If the group is not open users will need their request manually approved.'),
        ),
        migrations.AlterField(
            model_name='authgroup',
            name='public',
            field=models.BooleanField(default=False, help_text='Group is public. Any registered user is able to join this group, with visibility based on the other options set for this group.<br>Auth will not remove users from this group automatically when they are no longer authenticated.'),
        ),
        migrations.AlterField(
            model_name='authgroup',
            name='states',
            field=models.ManyToManyField(blank=True, help_text='States listed here will have the ability to join this group provided they have the proper permissions.<br>', related_name='valid_states', to='authentication.State'),
        ),
    ]
