# Generated by Django 1.10.1 on 2016-09-05 21:38

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AuthServicesInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ipboard_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('ipboard_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('xenforo_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('xenforo_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('forum_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('forum_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('jabber_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('jabber_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('mumble_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('mumble_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('teamspeak3_uid', models.CharField(blank=True, default=b'', max_length=254)),
                ('teamspeak3_perm_key', models.CharField(blank=True, default=b'', max_length=254)),
                ('discord_uid', models.CharField(blank=True, default=b'', max_length=254)),
                ('discourse_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('discourse_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('ips4_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('ips4_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('ips4_id', models.CharField(blank=True, default=b'', max_length=254)),
                ('smf_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('smf_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('market_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('market_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('pathfinder_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('pathfinder_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('main_char_id', models.CharField(blank=True, default=b'', max_length=64)),
                ('is_blue', models.BooleanField(default=False)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]
