# Generated by Django 3.1.4 on 2020-12-30 13:11

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AnalyticsIdentifier',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('identifier', models.UUIDField(default=uuid.uuid4, editable=False)),
            ],
        ),
        migrations.CreateModel(
            name='AnalyticsPath',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ignore_path', models.CharField(default='/example/', max_length=254)),
            ],
        ),
        migrations.CreateModel(
            name='AnalyticsTokens',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=254)),
                ('type', models.CharField(choices=[('GA-U', 'Google Analytics Universal'), ('GA-V4', 'Google Analytics V4')], max_length=254)),
                ('token', models.CharField(max_length=254)),
                ('send_page_views', models.BooleanField(default=False)),
                ('send_celery_tasks', models.BooleanField(default=False)),
                ('send_stats', models.BooleanField(default=False)),
                ('ignore_paths', models.ManyToManyField(blank=True, to='analytics.AnalyticsPath')),
            ],
        ),
    ]
