# Generated by Django 1.10.5 on 2017-02-04 06:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('groupmanagement', '0004_authgroup'),
    ]

    operations = [
        migrations.AddField(
            model_name='authgroup',
            name='public',
            field=models.BooleanField(default=False, help_text='Group is public. Any registered user is able to join this group, with visibility based on the other options set for this group.<br> Auth will not remove users from this group automatically when they are no longer authenticated.'),
        ),
    ]
