/*
CSS for allianceauth admin site
*/

/* styling for profile pic */
.img-circle {
    border-radius: 50%;
}
.column-user_profile_pic {
    width: 1px;
    white-space: nowrap;
}

/* tooltip */
.tooltip {
    position: relative ;
}
.tooltip:hover::after {
    content: attr(data-tooltip) ;
    position: absolute ;
    top: 1.1em ;
    left: 1em ;
    min-width: 200px ;
    border: 1px #808080 solid ;
    padding: 8px ;
    color: black ;
    background-color: rgb(255, 255, 204) ;
    z-index: 1 ;
}
