# Generated by Django 3.1.6 on 2021-03-23 13:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mumble', '0001_squashed_0011_auto_20201011_1009'),
    ]

    operations = [
        migrations.AddField(
            model_name='mumbleuser',
            name='last_connect',
            field=models.DateTimeField(blank=True, editable=False, help_text='Timestamp of the users Last Connection to Mumble', max_length=254, null=True, verbose_name='Last Connection'),
        ),
        migrations.AddField(
            model_name='mumbleuser',
            name='last_disconnect',
            field=models.DateTimeField(blank=True, editable=False, help_text='Timestamp of the users Last Disconnection to Mumble', max_length=254, null=True, verbose_name='Last Disconnection'),
        ),
        migrations.AddField(
            model_name='mumbleuser',
            name='release',
            field=models.TextField(blank=True, editable=False, help_text='The Mumble Release the user last authenticated with', max_length=254, null=True, verbose_name='Mumble Release'),
        ),
        migrations.AddField(
            model_name='mumbleuser',
            name='version',
            field=models.IntegerField(blank=True, editable=False, help_text='Client version. Major version in upper 16 bits, followed by 8 bits of minor version and 8 bits of patchlevel. Version 1.2.3 = 0x010203.', null=True, verbose_name='Mumble Version'),
        ),
    ]
