# Generated by Django 3.1.2 on 2021-01-08 13:54

from django.conf import settings
from django.db import migrations, models
import django.db.migrations.operations.special
import django.db.models.deletion

class Migration(migrations.Migration):

    replaces = [('mumble', '0001_initial'), ('mumble', '0002_auto_20161212_0100'), ('mumble', '0003_mumbleuser_user'), ('mumble', '0004_auto_20161214_1024'), ('mumble', '0005_mumbleuser_hashfn'), ('mumble', '0006_service_permissions'), ('mumble', '0007_not_null_user'), ('mumble', '0008_mumbleuser_display_name'), ('mumble', '0009_set_mumble_dissplay_names'), ('mumble', '0010_mumbleuser_certhash'), ('mumble', '0011_auto_20201011_1009')]

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='MumbleUser',
            fields=[
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, related_name='mumble', serialize=False, to=settings.AUTH_USER_MODEL)),
                ('username', models.CharField(max_length=254, unique=True)),
                ('pwhash', models.CharField(max_length=40)),
                ('groups', models.TextField(blank=True, null=True)),
            ],
            options={
                'db_table': 'services_mumbleuser',
            },
        ),
        migrations.AlterModelTable(
            name='mumbleuser',
            table=None,
        ),
        migrations.AddField(
            model_name='mumbleuser',
            name='hashfn',
            field=models.CharField(default='sha1', max_length=20),
        ),
        migrations.AlterField(
            model_name='mumbleuser',
            name='pwhash',
            field=models.CharField(max_length=80),
        ),
        migrations.AlterModelOptions(
            name='mumbleuser',
            options={'permissions': (('access_mumble', 'Can access the Mumble service'),)},
        ),
        migrations.AddField(
            model_name='mumbleuser',
            name='display_name',
            field=models.CharField(max_length=254, null=True),
        ),
        migrations.AlterField(
            model_name='mumbleuser',
            name='display_name',
            field=models.CharField(max_length=254, unique=True),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='mumbleuser',
            name='certhash',
            field=models.CharField(blank=True, editable=False, help_text='Hash of Mumble client certificate as presented when user authenticates', max_length=254, null=True, verbose_name='Certificate Hash'),
        ),
        migrations.AlterField(
            model_name='mumbleuser',
            name='pwhash',
            field=models.CharField(max_length=90),
        ),
    ]
