{% extends "timerboard/form.html" %}
{% load i18n %}

{% block page_title %}
    {% translate "Update Structure Timer" %}
{% endblock page_title %}

{% block page_header %}
    {% translate "Update Structure Timer" %}
{% endblock %}

{% block submit_button_text %}
    {% translate "Update Structure Timer" %}
{% endblock %}
