{% extends 'public/middle_box.html' %}
{% load bootstrap %}
{% load i18n %}
{% load static %}
{% block page_title %}{% translate "Register" %}{% endblock %}
{% block middle_box_content %}
    <form class="form-signin" role="form" action="" method="POST">
        {% csrf_token %}
        {{ form|bootstrap }}
        <br/>
        <button class="btn btn-lg btn-primary btn-block" type="submit">{% translate "Submit" %}</button>
        <br/>
    </form>
{% endblock %}
