{% extends 'allianceauth/base.html' %}
{% load i18n %}
{% block page_title %}{% translate "Fleet Participation" %}{% endblock %}

{% block content %}
<div class="col-lg-12">
    <h1 class="page-header text-center">{% translate "Character not found!" %}</h1>
    <div class="col-lg-12 container" id="example">
        <div class="row">
            <div class="col-lg-12">
                <div class="panel panel-default">
                    <div class="panel-heading">{{ character_name }}</div>
                    <div class="panel-body">
                        <div class="col-lg-2 col-sm-2">
                            <img class="ra-avatar img-responsive" src="{{ character_portrait_url }}">
                        </div>
                        <div class="col-lg-10 col-sm-2">
                            <div class="alert alert-danger" role="alert">{% translate "Character not registered!" %}</div>
                            {% translate "This character is not associated with an auth account." %} <a href=" {% url 'authentication:add_character' %}">{% translate "Add it here" %}</a> {% translate "before attempting to click fleet attendance links." %}
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </div>
</div>
{% endblock %}
