{% extends "allianceauth/base.html" %}
{% load static %}
{% load i18n %}

{% block page_title %}{% translate "Choose a Corp" %}{% endblock page_title %}
{% block content %}
    <div class="col-lg-12">
        <h1 class="page-header text-center">{% translate "Choose a Corp" %}</h1>
        {% if choices %}
            <div class="panel panel-primary">
                <div class="panel-heading">{% translate "Available Corps" %}</div>
                <table class="table table-responsive">
                    {% for choice in choices %}
                        <tr>
                            <td class="text-center">
                                <a href="{% url 'hrapplications:create_view' choice.0 %}" class="btn btn-primary" title="Apply">{{ choice.1 }}</a>
                            </td>
                        </tr>
                    {% endfor %}
                </table>
            </div>
        {% else %}
            <div class="alert alert-danger">{% translate "No corps are accepting applications at this time." %}</div>
        {% endif %}
    </div>
{% endblock content %}
