# Generated by Django 1.10.1 on 2016-09-10 20:20

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='eveapikeypair',
            name='error_count',
        ),
    ]
