{% load i18n %}
{% load auth_notifications %}

{% with unread_count=request.user|user_unread_notification_count %}
    {% if unread_count > 0 %}
        <a href="{% url 'notifications:list' %}">{% translate "Notifications" %}
            <span class="badge">{{ unread_count }}</span>
        </a>
    {% else %}
        <a href="{% url 'notifications:list' %}">
            <i class="far fa-bell"></i>
        </a>
    {% endif %}
{% endwith %}
