{% extends "allianceauth/base.html" %}
{% load bootstrap %}
{% load static %}
{% load i18n %}

{% block page_title %}{% translate "Create Fatlink" %}{% endblock page_title %}

{% block content %}
    <div class="col-lg-12">
        <h1 class="page-header text-center">{% translate "Create Fleet Operation" %}</h1>

        <div class="container-fluid">
            {% if badrequest %}
            <div class="alert alert-danger" role="alert">{% translate "Bad request!" %}</div>
            {% endif %}
            {% for message in errormessages %}
            <div class="alert alert-danger" role="alert">{{ message }}</div>
            {% endfor %}
            <div class="col-md-4 col-md-offset-4">
                <div class="row">
                        <form class="form-signin" role="form" action="" method="POST">
                            {% csrf_token %}
                            {{ form|bootstrap }}
                            <br/>
                            <button class="btn btn-lg btn-primary btn-block" type="submit" name="submit_fat">{% translate "Create fatlink" %}</button>
                        </form>
                </div>
            </div>
        </div>
    </div>

{% endblock content %}
