{% extends "allianceauth/base.html" %}
{% load bootstrap %}
{% load static %}
{% load i18n %}

{% block page_title %}{% translate "Jabber Broadcast" %}{% endblock page_title %}
{% block extra_css %}{% endblock extra_css %}

{% block content %}
    <div class="col-lg-12">
        <h1 class="page-header text-center">{% translate "Jabber Broadcast" %}</h1>

        <div class="container-fluid">
            <div class="col-md-4 col-md-offset-4">
                <div class="row">
                    {% if success %}
                        <div class="alert alert-success" role="alert">{% translate "Broadcast Sent!!" %}</div>
                    {% endif %}
                    <form class="form-signin" role="form" action="" method="POST" onsubmit="submitbutton.disabled = true; return true;">
                        {% csrf_token %}
                        {{ form|bootstrap }}
                        <br/>
                        <button class="btn btn-lg btn-primary btn-block" name="submitbutton" type="submit">{% translate "Broadcast" %}
                        </button>
                    </form>
                </div>
            </div>
        </div>
    </div>

{% endblock content %}
